@@ -13,7 +13,6 @@
 # License for the specific language governing permissions and limitations
 # under the License.
 
-import tempfile
 import uuid
 
 from keystone import config
@@ -21,6 +20,7 @@ from keystone import exception
 from keystone.openstack.common import jsonutils
 from keystone.policy.backends import rules
 from keystone import tests
+from keystone.tests.ksfixtures import temporaryfile
 from keystone.tests import test_v3
 
 
@@ -56,7 +56,8 @@ class IdentityTestProtectedCase(test_v3.RestfulTestCase):
         # file in each test to create the policies
         self.addCleanup(rules.reset)
         rules.reset()
-        _unused, self.tmpfilename = tempfile.mkstemp()
+        self.tempfile = self.useFixture(temporaryfile.SecureTempFile())
+        self.tmpfilename = self.tempfile.file_name
         self.config_fixture.config(policy_file=self.tmpfilename)
 
         # A default auth request we can use - un-scoped user token
@@ -65,6 +66,7 @@ class IdentityTestProtectedCase(test_v3.RestfulTestCase):
             password=self.user1['password'])
 
     def load_sample_data(self):
+        self._populate_default_domain()
         # Start by creating a couple of domains
         self.domainA = self.new_domain_ref()
         self.assignment_api.create_domain(self.domainA['id'], self.domainA)
@@ -76,25 +78,31 @@ class IdentityTestProtectedCase(test_v3.RestfulTestCase):
 
         # Now create some users, one in domainA and two of them in domainB
         self.user1 = self.new_user_ref(domain_id=self.domainA['id'])
-        self.user1['password'] = uuid.uuid4().hex
-        self.identity_api.create_user(self.user1['id'], self.user1)
+        password = uuid.uuid4().hex
+        self.user1['password'] = password
+        self.user1 = self.identity_api.create_user(self.user1)
+        self.user1['password'] = password
 
         self.user2 = self.new_user_ref(domain_id=self.domainB['id'])
-        self.user2['password'] = uuid.uuid4().hex
-        self.identity_api.create_user(self.user2['id'], self.user2)
+        password = uuid.uuid4().hex
+        self.user2['password'] = password
+        self.user2 = self.identity_api.create_user(self.user2)
+        self.user2['password'] = password
 
         self.user3 = self.new_user_ref(domain_id=self.domainB['id'])
-        self.user3['password'] = uuid.uuid4().hex
-        self.identity_api.create_user(self.user3['id'], self.user3)
+        password = uuid.uuid4().hex
+        self.user3['password'] = password
+        self.user3 = self.identity_api.create_user(self.user3)
+        self.user3['password'] = password
 
         self.group1 = self.new_group_ref(domain_id=self.domainA['id'])
-        self.identity_api.create_group(self.group1['id'], self.group1)
+        self.group1 = self.identity_api.create_group(self.group1)
 
         self.group2 = self.new_group_ref(domain_id=self.domainA['id'])
-        self.identity_api.create_group(self.group2['id'], self.group2)
+        self.group2 = self.identity_api.create_group(self.group2)
 
         self.group3 = self.new_group_ref(domain_id=self.domainB['id'])
-        self.identity_api.create_group(self.group3['id'], self.group3)
+        self.group3 = self.identity_api.create_group(self.group3)
 
         self.role = self.new_role_ref()
         self.assignment_api.create_role(self.role['id'], self.role)
@@ -372,6 +380,7 @@ class IdentityTestv3CloudPolicySample(test_v3.RestfulTestCase):
 
     def load_sample_data(self):
         # Start by creating a couple of domains
+        self._populate_default_domain()
         self.domainA = self.new_domain_ref()
         self.assignment_api.create_domain(self.domainA['id'], self.domainA)
         self.domainB = self.new_domain_ref()
@@ -383,22 +392,30 @@ class IdentityTestv3CloudPolicySample(test_v3.RestfulTestCase):
         # And our users
         self.cloud_admin_user = self.new_user_ref(
             domain_id=self.admin_domain['id'])
-        self.cloud_admin_user['password'] = uuid.uuid4().hex
-        self.identity_api.create_user(self.cloud_admin_user['id'],
-                                      self.cloud_admin_user)
+        password = uuid.uuid4().hex
+        self.cloud_admin_user['password'] = password
+        self.cloud_admin_user = (
+            self.identity_api.create_user(self.cloud_admin_user))
+        self.cloud_admin_user['password'] = password
         self.just_a_user = self.new_user_ref(domain_id=self.domainA['id'])
-        self.just_a_user['password'] = uuid.uuid4().hex
-        self.identity_api.create_user(self.just_a_user['id'], self.just_a_user)
+        password = uuid.uuid4().hex
+        self.just_a_user['password'] = password
+        self.just_a_user = self.identity_api.create_user(self.just_a_user)
+        self.just_a_user['password'] = password
         self.domain_admin_user = self.new_user_ref(
             domain_id=self.domainA['id'])
-        self.domain_admin_user['password'] = uuid.uuid4().hex
-        self.identity_api.create_user(self.domain_admin_user['id'],
-                                      self.domain_admin_user)
+        password = uuid.uuid4().hex
+        self.domain_admin_user['password'] = password
+        self.domain_admin_user = (
+            self.identity_api.create_user(self.domain_admin_user))
+        self.domain_admin_user['password'] = password
         self.project_admin_user = self.new_user_ref(
             domain_id=self.domainA['id'])
-        self.project_admin_user['password'] = uuid.uuid4().hex
-        self.identity_api.create_user(self.project_admin_user['id'],
-                                      self.project_admin_user)
+        password = uuid.uuid4().hex
+        self.project_admin_user['password'] = password
+        self.project_admin_user = (
+            self.identity_api.create_user(self.project_admin_user))
+        self.project_admin_user['password'] = password
 
         # The admin role and another plain role
         self.admin_role = {'id': uuid.uuid4().hex, 'name': 'admin'}
@@ -419,7 +436,7 @@ class IdentityTestv3CloudPolicySample(test_v3.RestfulTestCase):
                                          user_id=self.just_a_user['id'],
                                          domain_id=self.domainA['id'])
 
-        # Create a assign roles to the project
+        # Create and assign roles to the project
         self.project = self.new_project_ref(domain_id=self.domainA['id'])
         self.assignment_api.create_project(self.project['id'], self.project)
         self.assignment_api.create_grant(self.admin_role['id'],
@@ -645,3 +662,24 @@ class IdentityTestv3CloudPolicySample(test_v3.RestfulTestCase):
             domain_id=self.admin_domain['id'])
 
         self._test_domain_management()
+
+    def test_list_user_credentials(self):
+        self.credential_user = self.new_credential_ref(self.just_a_user['id'])
+        self.credential_api.create_credential(self.credential_user['id'],
+                                              self.credential_user)
+        self.credential_admin = self.new_credential_ref(
+            self.cloud_admin_user['id'])
+        self.credential_api.create_credential(self.credential_admin['id'],
+                                              self.credential_admin)
+
+        self.auth = self.build_authentication_request(
+            user_id=self.just_a_user['id'],
+            password=self.just_a_user['password'])
+        url = '/credentials?user_id=%s' % self.just_a_user['id']
+        self.get(url, auth=self.auth)
+        url = '/credentials?user_id=%s' % self.cloud_admin_user['id']
+        self.get(url, auth=self.auth,
+                 expected_status=exception.ForbiddenAction.code)
+        url = '/credentials'
+        self.get(url, auth=self.auth,
+                 expected_status=exception.ForbiddenAction.code)
