@@ -14,25 +14,21 @@
 
 import uuid
 
+from oslo.utils import timeutils
 import six
 
 from keystone import assignment
 from keystone.common import controller
 from keystone.common import dependency
-from keystone import config
+from keystone.common import validation
 from keystone import exception
-from keystone.openstack.common.gettextutils import _
+from keystone.i18n import _
+from keystone.models import token_model
 from keystone.openstack.common import log
-from keystone.openstack.common import timeutils
+from keystone.trust import schema
 
 
 LOG = log.getLogger(__name__)
-CONF = config.CONF
-
-
-def _trustor_only(context, trust, user_id):
-    if user_id != trust.get('trustor_user_id'):
-        raise exception.Forbidden()
 
 
 def _trustor_trustee_only(trust, user_id):
@@ -46,8 +42,8 @@ def _admin_trustor_only(context, trust, user_id):
         raise exception.Forbidden()
 
 
-@dependency.requires('assignment_api', 'identity_api', 'trust_api',
-                     'token_api')
+@dependency.requires('assignment_api', 'identity_api', 'token_provider_api',
+                     'trust_api')
 class TrustV3(controller.V3Controller):
     collection_name = "trusts"
     member_name = "trust"
@@ -64,16 +60,17 @@ class TrustV3(controller.V3Controller):
     def _get_user_id(self, context):
         if 'token_id' in context:
             token_id = context['token_id']
-            token = self.token_api.get_token(token_id)
-            user_id = token['user']['id']
-            return user_id
+            token_data = self.token_provider_api.validate_token(token_id)
+            token_ref = token_model.KeystoneToken(token_id=token_id,
+                                                  token_data=token_data)
+            return token_ref.user_id
         return None
 
     def get_trust(self, context, trust_id):
         user_id = self._get_user_id(context)
         trust = self.trust_api.get_trust(trust_id)
         if not trust:
-            raise exception.TrustNotFound(trust_id)
+            raise exception.TrustNotFound(trust_id=trust_id)
         _trustor_trustee_only(trust, user_id)
         self._fill_in_roles(context, trust,
                             self.assignment_api.list_roles())
@@ -123,16 +120,13 @@ class TrustV3(controller.V3Controller):
         return trust_roles
 
     @controller.protected()
+    @validation.validated(schema.trust_create, 'trust')
     def create_trust(self, context, trust=None):
         """Create a new trust.
 
         The user creating the trust must be the trustor.
 
         """
-
-        # TODO(ayoung): instead of raising ValidationError on the first
-        # problem, return a collection of all the problems.
-
         # Explicitly prevent a trust token from creating a new trust.
         auth_context = context.get('environment',
                                    {}).get('KEYSTONE_AUTH_CONTEXT', {})
@@ -144,44 +138,58 @@ class TrustV3(controller.V3Controller):
         if not trust:
             raise exception.ValidationError(attribute='trust',
                                             target='request')
-        if trust.get('project_id') and not trust.get('roles'):
+        if trust.get('project_id'):
+            self._require_role(trust)
+        self._require_user_is_trustor(context, trust)
+        self._require_trustee_exists(trust['trustee_user_id'])
+        all_roles = self.assignment_api.list_roles()
+        clean_roles = self._clean_role_list(context, trust, all_roles)
+        self._require_trustor_has_role_in_project(trust, clean_roles)
+        trust['expires_at'] = self._parse_expiration_date(
+            trust.get('expires_at'))
+        trust_id = uuid.uuid4().hex
+        new_trust = self.trust_api.create_trust(trust_id, trust, clean_roles)
+        self._fill_in_roles(context, new_trust, all_roles)
+        return TrustV3.wrap_member(context, new_trust)
+
+    def _require_trustee_exists(self, trustee_user_id):
+        self.identity_api.get_user(trustee_user_id)
+
+    def _require_user_is_trustor(self, context, trust):
+        user_id = self._get_user_id(context)
+        if user_id != trust.get('trustor_user_id'):
+            raise exception.Forbidden(
+                _("The authenticated user should match the trustor."))
+
+    def _require_role(self, trust):
+        if not trust.get('roles'):
             raise exception.Forbidden(
                 _('At least one role should be specified.'))
+
+    def _get_user_role(self, trust):
+        if not self._attribute_is_empty(trust, 'project_id'):
+            return self.assignment_api.get_roles_for_user_and_project(
+                trust['trustor_user_id'], trust['project_id'])
+        else:
+            return []
+
+    def _require_trustor_has_role_in_project(self, trust, clean_roles):
+        user_roles = self._get_user_role(trust)
+        for trust_role in clean_roles:
+            matching_roles = [x for x in user_roles
+                              if x == trust_role['id']]
+            if not matching_roles:
+                raise exception.RoleNotFound(role_id=trust_role['id'])
+
+    def _parse_expiration_date(self, expiration_date):
+        if expiration_date is None:
+            return None
+        if not expiration_date.endswith('Z'):
+            expiration_date += 'Z'
         try:
-            user_id = self._get_user_id(context)
-            _trustor_only(context, trust, user_id)
-            #confirm that the trustee exists
-            self.identity_api.get_user(trust['trustee_user_id'])
-            all_roles = self.assignment_api.list_roles()
-            clean_roles = self._clean_role_list(context, trust, all_roles)
-            if trust.get('project_id'):
-                user_role = self.assignment_api.get_roles_for_user_and_project(
-                    user_id,
-                    trust['project_id'])
-            else:
-                user_role = []
-            for trust_role in clean_roles:
-                matching_roles = [x for x in user_role
-                                  if x == trust_role['id']]
-                if not matching_roles:
-                    raise exception.RoleNotFound(role_id=trust_role['id'])
-            if trust.get('expires_at') is not None:
-                if not trust['expires_at'].endswith('Z'):
-                    trust['expires_at'] += 'Z'
-                try:
-                    trust['expires_at'] = (timeutils.parse_isotime
-                                           (trust['expires_at']))
-                except ValueError:
-                    raise exception.ValidationTimeStampError()
-            trust_id = uuid.uuid4().hex
-            new_trust = self.trust_api.create_trust(trust_id,
-                                                    trust,
-                                                    clean_roles)
-            self._fill_in_roles(context, new_trust, all_roles)
-            return TrustV3.wrap_member(context, new_trust)
-        except KeyError as e:
-            raise exception.ValidationError(attribute=e.args[0],
-                                            target='trust')
+            return timeutils.parse_isotime(expiration_date)
+        except ValueError:
+            raise exception.ValidationTimeStampError()
 
     @controller.protected()
     def list_trusts(self, context):
@@ -219,19 +227,17 @@ class TrustV3(controller.V3Controller):
     def delete_trust(self, context, trust_id):
         trust = self.trust_api.get_trust(trust_id)
         if not trust:
-            raise exception.TrustNotFound(trust_id)
+            raise exception.TrustNotFound(trust_id=trust_id)
 
         user_id = self._get_user_id(context)
         _admin_trustor_only(context, trust, user_id)
         self.trust_api.delete_trust(trust_id)
-        userid = trust['trustor_user_id']
-        self.token_api.delete_tokens(userid, trust_id=trust_id)
 
     @controller.protected()
     def list_roles_for_trust(self, context, trust_id):
         trust = self.get_trust(context, trust_id)['trust']
         if not trust:
-            raise exception.TrustNotFound(trust_id)
+            raise exception.TrustNotFound(trust_id=trust_id)
         user_id = self._get_user_id(context)
         _trustor_trustee_only(trust, user_id)
         return {'roles': trust['roles'],
@@ -242,7 +248,7 @@ class TrustV3(controller.V3Controller):
         """Checks if a role has been assigned to a trust."""
         trust = self.trust_api.get_trust(trust_id)
         if not trust:
-            raise exception.TrustNotFound(trust_id)
+            raise exception.TrustNotFound(trust_id=trust_id)
         user_id = self._get_user_id(context)
         _trustor_trustee_only(trust, user_id)
         if not any(role['id'] == role_id for role in trust['roles']):
